package com.portalmedia.embarc.gui.helper;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import org.dizitart.no2.objects.Cursor;

import com.google.cloud.Crc32c;
import com.portalmedia.embarc.database.DBService;
import com.portalmedia.embarc.gui.model.DatabaseSummary;
import com.portalmedia.embarc.gui.model.FileInformationViewModel;
import com.portalmedia.embarc.gui.model.SelectedFilesSummary;
import com.portalmedia.embarc.parser.MetadataColumn;
import com.portalmedia.embarc.parser.dpx.DPXColumn;
import com.portalmedia.embarc.parser.dpx.DPXFileInformation;
import com.portalmedia.embarc.parser.dpx.DPXMetadata;
import com.portalmedia.embarc.parser.dpx.DPXService;
import com.portalmedia.embarc.validation.ValidationRuleSetEnum;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

/**
 * Method for handling files to/from the database and other convenience methods
 * needed for table view in UI.
 *
 * @author PortalMedia
 * @since 2019-05-01
 **/
public class DPXFileListHelper {
	static DBService<FileInformationViewModel> dbService = new DBService<>(FileInformationViewModel.class);

	private static int current_id = 0;
	private static boolean selectAll = false;
	public static boolean sortAsc = true;

	private static void AddFileInformationViewModel(DPXFileInformation f) {
		final FileInformationViewModel fivm = toFileInformationViewModel(f);
		DatabaseSummary.addFile(fivm);
		dbService.add(fivm);
	}

	public static <T> Boolean addFileToDatabase(String file) {
		if (file != null) {
			AddFileInformationViewModel(createDPXFileInformation(file));
		}
		return true;
	}

	public static <T> DPXFileInformation createDPXFileInformation(String filePath) {
		final File file = new File(filePath);
		final String name = file.getName();
		final String path = file.getPath();

		final DPXFileInformation fileInfo = new DPXFileInformation();
		fileInfo.setName(name);
		fileInfo.setPath(path);

		current_id++;
		fileInfo.setId(current_id);

		try {
			final DPXService service = new DPXService(file.getPath());
			fileInfo.setFileData(service.readFile());
		} catch (final Exception e) {
			System.out.println("Failed to parse file metadata");
			e.printStackTrace();
		}

		return fileInfo;
	}

	public static SelectedFilesSummary createSelectedFilesSummary(List<FileInformationViewModel> selectedRows) {

		if (selectAll) {
			final org.dizitart.no2.objects.Cursor<FileInformationViewModel> files = dbService.getAllCursors();
			SelectedFilesSummary selectedFilesSummary = SelectedFilesSummary
					.create(new ArrayList<FileInformationViewModel>());
			for (final FileInformationViewModel fi : files) {
				selectedFilesSummary = SelectedFilesSummary.append(fi);
				if (!selectedFilesSummary.hasPossibleColumnsLeft()) {
					break;
				}
			}
			return selectedFilesSummary;
		} else {
			return SelectedFilesSummary.create(selectedRows);
		}
	}

	public static SelectedFilesSummary createSelectedFilesSummaryAll() {

		SelectedFilesSummary selectedFilesSummary = SelectedFilesSummary
				.create(new ArrayList<FileInformationViewModel>());
		for (final FileInformationViewModel fi : dbService.getAllCursors()) {
			selectedFilesSummary = SelectedFilesSummary.append(fi);
			if (!selectedFilesSummary.hasPossibleColumnsLeft()) {
				break;
			}
		}
		return selectedFilesSummary;

	}

	public static <T> Boolean deleteFileFromDB(int id) {
		System.out.println("Deleting file: " + id);
		try {
			dbService.delete(id);
		} catch (final Exception e) {
			System.out.println("Failed to delete file");
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public static void deleteSelectedRows(List<FileInformationViewModel> selectedRows) {
		if (selectAll) {
			System.out.println("Select all delete");
			dbService.deleteAll();
			DatabaseSummary.deleteAll();
		} else {
			for (final FileInformationViewModel fivm : selectedRows) {
				dbService.delete(Integer.parseInt(fivm.getProp("id")));
				DatabaseSummary.removeFile(fivm);
			}
		}
	}

	public static Cursor<FileInformationViewModel> getAllFiles(boolean editedOnly) {
		return editedOnly ? dbService.getEditedCursors() : dbService.getAllCursors();
	}

	public static byte[] getBytesFromFile(String filePath, int start) throws IOException {
		final File f = new File(filePath);
		final byte[] allBytes = Files.readAllBytes(f.toPath());
		final byte[] imageData = Arrays.copyOfRange(allBytes, start, allBytes.length);
		return imageData;
	}

	public static String getCrc32Hash(byte[] bytes) {
		try {
			final Crc32c c = new Crc32c();
			c.update(bytes, 0, bytes.length);
			return String.format("%02x", c.getValue());
		} catch (final Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return "";
	}

	private static int getErrorCount(Set<ValidationRuleSetEnum> rules) {
		final long startTime = System.currentTimeMillis();
		final int size = dbService.getErrorCount(rules);
		System.out.println("Baseline getting error count " + (System.currentTimeMillis() - startTime) + "ms " + size);
		return size;
	}

	public static ObservableList<FileInformationViewModel> getObservableFileList() {
		return FXCollections.observableArrayList(dbService.getAll());
	}

	public static long getSelectedFileCount(ObservableList<FileInformationViewModel> selectedRows,
			Set<ValidationRuleSetEnum> rules, boolean filteredByErrors) {
		if (selectAll && filteredByErrors) {
			return getErrorCount(rules);
		}
		if (selectAll) {
			return dbService.getSize();
		} else {
			return selectedRows.size();
		}
	}

	public static int getTotalFiles() {
		return (int) dbService.getSize();
	}

	public static boolean selectAllSelected() {
		return selectAll;
	}

	public static void setSelectAll(boolean value) {
		selectAll = value;
	}

	private static FileInformationViewModel toFileInformationViewModel(DPXFileInformation f) {
		final FileInformationViewModel fivm = new FileInformationViewModel();
		fivm.setProp("name", f.getName());
		fivm.setProp("path", f.getPath());
		fivm.setId(f.getId());
		final DPXMetadata dpx = f.getFileData();
		final LinkedHashMap<DPXColumn, MetadataColumn> columns = dpx.getMetadataHashMap();

		for (final DPXColumn key : columns.keySet()) {
			final MetadataColumn c = columns.get(key);
			fivm.setProp(key, c.toString());
		}
		return fivm;
	}

	public static void updateName(FileInformationViewModel fivm) {
		fivm.autoPopulateName();
		dbService.update(fivm);
		DatabaseSummary.updateFile(fivm);
	}

	public static void updateValues(HashMap<DPXColumn, String> changedValues, FileInformationViewModel file) {
		for (final DPXColumn column : changedValues.keySet()) {
			file.setProp(column, changedValues.get(column));
		}
		file.setEdited(true);
		dbService.update(file);
		DatabaseSummary.updateFile(file);
	}

}
