package com.portalmedia.embarc.parser.dpx;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.nio.ByteOrder;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import com.portalmedia.embarc.gui.model.FileInformationViewModel;
import com.portalmedia.embarc.parser.BinaryFileReader;
import com.portalmedia.embarc.parser.ByteOrderEnum;
import com.portalmedia.embarc.parser.MetadataColumn;
import com.portalmedia.embarc.parser.ValueToBytesHelper;

/**
 * Service to read from and write to a DPX files
 * 
 * @author PortalMedia
 * @since   2019-05-01
 **/
public class DPXService {
	String filePath;
	public DPXService(String filePath) throws FileNotFoundException{
		this.filePath = filePath;
	}
	public DPXMetadata readFile() throws Exception
	{
		BinaryFileReader dpxFile = new BinaryFileReader(filePath);
		
		// Default byte order
		ByteOrderEnum byteOrder = ByteOrderEnum.BIG;
		
		// Create a new object to hold the metadata
		DPXMetadata dpxData = new DPXMetadata();
		
		// Iterate through each column in the File Information Header
		for(DPXColumn c : DPXSection.FILE_INFORMATION_HEADER.getColumns())
		{
			// Read the section of bytes
			byte[] bytes = dpxFile.readBytes(c.getLength());
			
			// Store this data in a metadata column
			MetadataColumn mdColumn = DPXMetadataColumnFactory.create(c, bytes, byteOrder);

			// If the magic number is "SDPX", we know the rest of the file is big endian
			if(c==DPXColumn.MAGIC_NUMBER)
			{
				if(mdColumn.toString().equals("SDPX")) {
				} else if(mdColumn.toString().equals("XPDS")) {
					byteOrder = ByteOrderEnum.LITTLE;
					mdColumn = DPXMetadataColumnFactory.create(DPXColumn.MAGIC_NUMBER, bytes, byteOrder);
				} else throw new IllegalArgumentException("Invalid magic number");
			}
			
			// Store the new column
			dpxData.setValue(mdColumn);
		}
		
		// Iterate through each column in the Image Information Header
		for(DPXColumn c : DPXSection.IMAGE_INFORMATION_HEADER.getColumns())
		{
			MetadataColumn mdColumn = DPXMetadataColumnFactory.create(c, dpxFile.readBytes(c.getLength()), byteOrder);
			dpxData.setValue(mdColumn);
		}

		// Iterate through each column in the Image Source Information Header
		for(DPXColumn c : DPXSection.IMAGE_SOURCE_INFORMATION_HEADER.getColumns())
		{
			MetadataColumn mdColumn = DPXMetadataColumnFactory.create(c, dpxFile.readBytes(c.getLength()), byteOrder);
			dpxData.setValue(mdColumn);
		}

		// Iterate through each column in the Motion Picture Film Information Header
		for(DPXColumn c : DPXSection.MOTION_PICTURE_FILM_INFORMATION_HEADER.getColumns())
		{
			MetadataColumn mdColumn = DPXMetadataColumnFactory.create(c, dpxFile.readBytes(c.getLength()), byteOrder);
			dpxData.setValue(mdColumn);
		}

		// Iterate through each column in the Television Information Header
		for(DPXColumn c : DPXSection.TELEVISION_INFORMATION_HEADER.getColumns())
		{
			MetadataColumn mdColumn = DPXMetadataColumnFactory.create(c, dpxFile.readBytes(c.getLength()), byteOrder);
			dpxData.setValue(mdColumn);
		}

		// Get the length of the User Identification Field
	    int userIdLength = DPXColumn.USER_IDENTIFICATION.getLength();
	    
	    // User defined data length is the total specified in the User Defined Header Length column minus the length of the user identification length
 		int lengthOfUserData = Math.max(dpxData.getColumn(DPXColumn.USER_DEFINED_HEADER_LENGTH).getInt()-userIdLength,0);
 		
 		// Read the user identification data
		MetadataColumn user = DPXMetadataColumnFactory.create(DPXColumn.USER_IDENTIFICATION, dpxFile.readBytes(userIdLength), byteOrder);
		dpxData.setValue(user);
		
		// Read the user defined data.  
		MetadataColumn userDefinedData = DPXMetadataColumnFactory.create(DPXColumn.USER_DEFINED_DATA, dpxFile.readBytes(lengthOfUserData), byteOrder);
		dpxData.setValue(userDefinedData);
		
		// close the file
		dpxFile.close();
		return dpxData;
	}
	public static boolean writeFile(FileInformationViewModel file, String inputPath, String outputPath) throws Exception{
	    InputStream is = null;
        // Original value of offset to image data - needed for checksum
        String imageDataStartString = file.getProp(DPXColumn.OFFSET_TO_IMAGE_DATA);
        int imageDataStart = Integer.parseInt(imageDataStartString);
        
        // Create a temp file.  
        String tempPath = outputPath + ".tmp";
	    try {
			try (FileOutputStream fos = new FileOutputStream(tempPath)) {
				// Determine byte ordering
				String magicNumber = file.getProp(DPXColumn.MAGIC_NUMBER);
				ByteOrder byteOrder = magicNumber.equals("SDPX") ? ByteOrder.BIG_ENDIAN : ByteOrder.LITTLE_ENDIAN;
				
				// Get the length of the user defined data
				String userDefinedDataLengthStr = file.getProp(DPXColumn.USER_DEFINED_HEADER_LENGTH);
				int userDefinedHeaderLength = Integer.parseInt(userDefinedDataLengthStr);
				
				// Iterate through each column
				for(DPXColumn column : DPXColumn.values()) {
					String value = "";
					int length = column.getLength();
					if(column == DPXColumn.USER_DEFINED_DATA) {
						length = userDefinedHeaderLength - DPXColumn.USER_IDENTIFICATION.getLength();
					}
						
					if(column==DPXColumn.USER_DEFINED_HEADER_LENGTH) {
						int x = userDefinedHeaderLength;
						value = String.valueOf(x);
					}
					else if(column==DPXColumn.OFFSET_TO_IMAGE_DATA) value = String.valueOf(file.getOffsetToImageData());
					else value = file.getProp(column);
					
					byte[] convertedValue = ValueToBytesHelper.ToTypedValue(column, value, length, byteOrder);
					fos.write(convertedValue);
				}
				
				// Read the image data from the original file
		        is = new FileInputStream(inputPath);
		        byte[] buffer = new byte[1024];
		        int length;
		        is.skip(imageDataStart);
		        
		        // Write the image data to the temp file
		        while ((length = is.read(buffer)) > 0) {
		            fos.write(buffer, 0, length);
		        }
	        	is.close();
	        	fos.close();
	        	
	        	// Copy the temp file to the final destination
	        	Path tempFile = new File(tempPath).toPath();
		        try {
		        	Files.copy(tempFile, new File(outputPath).toPath(), StandardCopyOption.REPLACE_EXISTING);
		        } catch(Exception ex) {
					ex.printStackTrace();
		        }
		        try {
		        	// cleanup temp file
		        	Files.delete(tempFile);
		        	return true;
		        } catch(Exception ex) {
					ex.printStackTrace();
		        }
		        
			} catch(Exception ex) {
				// Cleanup on error
				Path tempFile = new File(tempPath).toPath();
				if(Files.exists(tempFile)) Files.delete(tempFile);
				ex.printStackTrace();
		    	return false;
			}
			
	    } 
	    catch(Exception ex) {
	    	ex.printStackTrace();
	    	return false;
	    }
	    finally {
	        is.close();
	    }
	    return false;
	}
	
}

